@@ -81,7 +81,7 @@ int echo_builtin(int argc, void **argv)
      *   SYSV, XSI, BSD, GNU, ZSH, DASH, RAW
      * But we only care about the first character of it. */
     nonewline = false;
-    echo_style = getvar(VAR_ECHO_STYLE);
+    echo_style = getvar(L VAR_ECHO_STYLE);
     switch (echo_style ? echo_style[0] : L'\0') {
 	case L'S':  case L's':  case L'X':  case L'x':  default:
 	    escape = true, noption = false, eoption = false;
@@ -234,7 +234,7 @@ print_char:
 const char echo_help[] = Ngt(
 "echo - prints arguments\n"
 "\techo [string...]\n"
-"Prints <string>s, followed by a newline. Adjacent <string>s are separated by\n"
+"Prints <string>s, followed by a newline. The <string>s are each separated by\n"
 "a space. This command does not recognize any options except described below.\n"
 "\n"
 "The behavior of \"echo\" depends on the value of $ECHO_STYLE variable:\n"
@@ -407,7 +407,7 @@ put_char:
 	default:  normal:
 	    {
 		if (!sb_wccat(&buf, *format, &state)) {
-		    xerror(0, Ngt("unexpected error"));
+		    xerror(EILSEQ, Ngt("unexpected error"));
 		    sb_destroy(&buf);
 		    return false;
 		}
@@ -676,14 +676,16 @@ const char printf_help[] = Ngt(
 "  %e    floating point number with exponent (lowercase)\n"
 "  %E    floating point number with exponent (uppercase)\n"
 "  %g    %f or %e, automatically selected\n"
-"  %G    %f or %E, automatically selected\n"
+"  %G    %F or %E, automatically selected\n"
 "  %c    first character of string\n"
 "  %s    string\n"
 "  %b    string (escape sequences recognized as \"echo\")\n"
 "  %%    %\n"
 "\n"
 "Flags, field width, and precision can be specified in this order between\n"
-"`%' and the conversion specifier.\n"
+"`%' and the conversion specifier. For example, `%08.3f' is a specifier for a\n"
+"floating point number with the `0' flag, the field width of 8, and the\n"
+"precision of 3.\n"
 "\n"
 "Escape sequences:\n"
 "  \\a     alert (bell)\n"
