@@ -350,7 +350,7 @@ void exec_for(const command_T *c, bool finally_exit)
 	    goto finish;
 	}
     } else {
-	struct get_variable v = get_variable("@");
+	struct get_variable v = get_variable(L"@");
 	assert(v.type == GV_ARRAY && v.values != NULL);
 	words = duparray(v.values, copyaswcs);
 	count = plcount(words);
@@ -368,12 +368,11 @@ void exec_for(const command_T *c, bool finally_exit)
     } else (void) 0
 
     int i;
-    for (i = 0; i < count; ) {
+    for (i = 0; i < count; i++) {
 	if (!set_variable(c->c_forname, words[i],
 		    posixly_correct ? SCOPE_GLOBAL : SCOPE_LOCAL, false))
 	    goto done;
-	i++;
-	exec_and_or_lists(c->c_forcmds, finally_exit && i == count);
+	exec_and_or_lists(c->c_forcmds, finally_exit && i + 1 == count);
 	CHECK_LOOP;
     }
 
@@ -384,9 +383,9 @@ done:
     if (count == 0 && c->c_forcmds)
 	laststatus = Exit_SUCCESS;
 finish:
+    execinfo.loopnest--;
     if (finally_exit)
 	exit_shell();
-    execinfo.loopnest--;
 }
 
 /* Executes a while/until command. */
@@ -409,9 +408,9 @@ void exec_while(const command_T *c, bool finally_exit)
 
     laststatus = status;
 done:
+    execinfo.loopnest--;
     if (finally_exit)
 	exit_shell();
-    execinfo.loopnest--;
 }
 #undef CHECK_LOOP
 
@@ -863,7 +862,7 @@ void search_command(
 
     /* search builtins and functions. */
     const builtin_T *bi = (type & sct_builtin) ? get_builtin(name) : NULL;
-    command_T *funcbody = (type & sct_function) ? get_function(name) : NULL;
+    command_T *funcbody = (type & sct_function) ? get_function(wname) : NULL;
     if (bi && bi->type == BI_SPECIAL) {
 	ci->type = specialbuiltin;
 	ci->ci_builtin = bi->body;
@@ -933,7 +932,7 @@ bool assignment_is_temporary(enum cmdtype_T type)
 bool including_path_assignment(const assign_T *a)
 {
     while (a) {
-	if (strcmp(a->a_name, VAR_PATH) == 0)
+	if (wcscmp(a->a_name, L VAR_PATH) == 0)
 	    return true;
 	a = a->next;
     }
@@ -1286,7 +1285,7 @@ int return_builtin(int argc, void **argv)
 const char return_help[] = Ngt(
 "return - return from function\n"
 "\treturn [n]\n"
-"Exits the currently executed function or script file with the exit status\n"
+"Exits the currently executing function or script file with the exit status\n"
 "of <n>. If <n> is not specified, it defaults to the exit status of the last\n"
 "executed command. <n> should be between 0 and 255 inclusive.\n"
 );
@@ -1350,14 +1349,14 @@ int break_builtin(int argc, void **argv)
 const char break_help[] = Ngt(
 "break - exit loop\n"
 "\tbreak [n]\n"
-"Exits the currently executed for, while or until loop.\n"
+"Exits the currently executing for, while or until loop.\n"
 "If <n> is specified, exits the <n>th outer loop.\n"
 );
 
 const char continue_help[] = Ngt(
 "continue - continue loop\n"
 "\tcontinue [n]\n"
-"Returns to the top of the currently executed for, while or until loop.\n"
+"Returns to the top of the currently executing for, while or until loop.\n"
 "If <n> is specified, returns to that of the <n>th outer loop.\n"
 );
 
@@ -1413,7 +1412,7 @@ int dot_builtin(int argc, void **argv)
 
     char *mbsfilename = malloc_wcstombs(filename);
     if (!mbsfilename) {
-	xerror(0, Ngt("unexpected error"));
+	xerror(EILSEQ, Ngt("unexpected error"));
 	return Exit_ERROR;
     }
 
@@ -1546,7 +1545,7 @@ int exec_builtin_2(int argc, void **argv, const wchar_t *as, bool clear)
     for (int i = 0; i < argc; i++) {
 	args[i] = malloc_wcstombs(ARGV(i));
 	if (!args[i]) {
-	    xerror(0, Ngt("cannot convert wide characters into "
+	    xerror(EILSEQ, Ngt("cannot convert wide characters into "
 			"multibyte characters: replaced with empty string"));
 	    args[i] = xstrdup("");
 	}
@@ -1571,9 +1570,9 @@ int exec_builtin_2(int argc, void **argv, const wchar_t *as, bool clear)
 	
 	pl_init(&list);
 	for (assign_T *assign = last_assign; assign; assign = assign->next) {
-	    const char *envval = getenv(assign->a_name);
+	    const wchar_t *val = getvar(assign->a_name);
 	    pl_add(&list, malloc_printf(
-			"%s=%s", assign->a_name, envval ? envval : ""));
+			"%ls=%ls", assign->a_name, val ? val : L""));
 	}
 	envs = (char **) pl_toary(&list);
     } else {
@@ -1584,7 +1583,7 @@ int exec_builtin_2(int argc, void **argv, const wchar_t *as, bool clear)
 	freelater = args[0];
 	args[0] = malloc_wcstombs(as);
 	if (!args[0]) {
-	    xerror(0, Ngt("cannot convert wide characters into "
+	    xerror(EILSEQ, Ngt("cannot convert wide characters into "
 			"multibyte characters: replaced with empty string"));
 	    args[0] = xstrdup("");
 	}
@@ -1840,7 +1839,7 @@ void print_command_absolute_path(
 	return;
     }
 
-    const wchar_t *wpwd = getvar(VAR_PWD);
+    const wchar_t *wpwd = getvar(L VAR_PWD);
     char *pwd = NULL;
     if (wpwd) {
 	pwd = malloc_wcstombs(wpwd);
@@ -1883,6 +1882,7 @@ const char command_help[] = Ngt(
 "printed and the exit status is non-zero.\n"
 "With the -V (--verbose-identify) option, the command is identified in the\n"
 "same way but the result is printed verbosely in a human-readable form.\n"
+"In the POSIXly correct mode, the -b and -B options are not available.\n"
 );
 
 const char type_help[] = Ngt(
