@@ -168,7 +168,7 @@ bool expand_line(void *const *restrict args,
 	if (!expand_word_and_split(*args, &list1)) {
 	    if (!is_interactive)
 		exit_shell_with_status(Exit_EXPERROR);
-	    recfree(pl_toary(&list1), free);
+	    pl_destroy(pl_clear(&list1, free));
 	    return false;
 	}
 	args++;
@@ -199,8 +199,8 @@ bool expand_word_and_split(const wordunit_T *restrict w, plist_T *restrict list)
 
     /* four expansions (w -> list1) */
     if (!expand_word(w, tt_single, false, &valuelist1, &splitlist1)) {
-	recfree(pl_toary(&valuelist1), free);
-	recfree(pl_toary(&splitlist1), free);
+	pl_destroy(pl_clear(&valuelist1, free));
+	pl_destroy(pl_clear(&splitlist1, free));
 	return false;
     }
 
@@ -238,15 +238,15 @@ wchar_t *expand_single(const wordunit_T *arg, tildetype_T tilde)
     if (!expand_word(arg, tilde, false, &list, NULL)) {
 	if (!is_interactive)
 	    exit_shell_with_status(Exit_EXPERROR);
-	recfree(pl_toary(&list), free);
+	pl_destroy(pl_clear(&list, free));
 	return NULL;
     }
     if (list.length != 1) {
 	/* concatenate multiple words to a single */
-	const wchar_t *ifs = getvar(VAR_IFS);
+	const wchar_t *ifs = getvar(L VAR_IFS);
 	wchar_t padding[] = { ifs ? ifs[0] : L' ', L'\0' };
 	result = joinwcsarray(list.contents, padding);
-	recfree(pl_toary(&list), free);
+	pl_destroy(pl_clear(&list, free));
     } else {
 	result = list.contents[0];
 	pl_destroy(&list);
@@ -274,7 +274,7 @@ char *expand_single_with_glob(const wordunit_T *arg, tildetype_T tilde)
 noglob:
 	result = realloc_wcstombs(unescapefree(exp));
 	if (!result)
-	    xerror(0, Ngt("cannot convert multibyte characters "
+	    xerror(EILSEQ, Ngt("cannot convert multibyte characters "
 			"into wide characters: replaced with empty string"));
     } else {
 	plist_T list;
@@ -285,7 +285,7 @@ noglob:
 	    result = list.contents[0];
 	    pl_destroy(&list);
 	} else {
-	    recfree(pl_toary(&list), free);
+	    pl_destroy(pl_clear(&list, free));
 	    if (posixly_correct) {
 		goto noglob;
 	    } else {
@@ -554,7 +554,7 @@ wchar_t *expand_tilde(const wchar_t **ss, bool hasnextwordunit, tildetype_T tt)
     username = xwcsndup(s, usernamelen);
     if (username[0] == L'\0') {
 	/* empty user name: use $HOME */
-	home = getvar(VAR_HOME);
+	home = getvar(L VAR_HOME);
 	goto finish;
     } else if (wcspbrk(username, L"\"'\\") != 0) {
 	/* don't expand if the user name is quoted */
@@ -565,7 +565,7 @@ wchar_t *expand_tilde(const wchar_t **ss, bool hasnextwordunit, tildetype_T tt)
 	size_t index = index;
 	if (username[0] == L'+') {
 	    if (username[1] == L'\0') {
-		home = getvar(VAR_PWD);
+		home = getvar(L VAR_PWD);
 		goto finish;
 #if YASH_ENABLE_DIRSTACK
 	    } else if (parse_dirstack_index(username, &index, &home, false)
@@ -575,7 +575,7 @@ wchar_t *expand_tilde(const wchar_t **ss, bool hasnextwordunit, tildetype_T tt)
 	    }
 	} else if (username[0] == L'-') {
 	    if (username[1] == L'\0') {
-		home = getvar(VAR_OLDPWD);
+		home = getvar(L VAR_OLDPWD);
 		goto finish;
 #if YASH_ENABLE_DIRSTACK
 	    } else if (parse_dirstack_index(username, &index, &home, false)
@@ -654,7 +654,7 @@ bool expand_param(const paramexp_T *p, bool indq, struct expand_word_T *e)
 	plist_T plist;
 	pl_init(&plist);
 	if (!expand_word(p->pe_nest, tt_none, true, &plist, NULL)) {
-	    recfree(pl_toary(&plist), free);
+	    pl_destroy(pl_clear(&plist, free));
 	    return false;
 	}
 	v.type = (plist.length == 1) ? GV_SCALAR : GV_ARRAY;
@@ -762,12 +762,12 @@ subst:
 		xerror(0, Ngt("invalid assignment in parameter expansion"));
 		return false;
 	    } else if (!is_name(p->pe_name)) {
-		xerror(0, Ngt("cannot assign to `%s' in parameter expansion"),
+		xerror(0, Ngt("cannot assign to `%ls' in parameter expansion"),
 			p->pe_name);
 		return false;
 	    } else if ((v.type == GV_ARRAY_CONCAT)
 		    || (v.type == GV_ARRAY && startindex + 1 != endindex)) {
-		xerror(0, Ngt("cannot assign to array range `%s[...]' "
+		xerror(0, Ngt("cannot assign to array range `%ls[...]' "
 			    "in parameter expansion"),
 			p->pe_name);
 		return false;
@@ -806,7 +806,7 @@ subst:
 
     if (shopt_nounset && unset) {
 	recfree(list, free);
-	xerror(0, Ngt("%s: parameter not set"), p->pe_name);
+	xerror(0, Ngt("%ls: parameter not set"), p->pe_name);
 	return false;
     }
 
@@ -839,7 +839,7 @@ subst:
 
     /* concatenate `list' elements */
     if (concat || !indq) {
-	const wchar_t *ifs = getvar(VAR_IFS);
+	const wchar_t *ifs = getvar(L VAR_IFS);
 	wchar_t padding[] = { ifs ? ifs[0] : L' ', L'\0' };
 	wchar_t *chain = joinwcsarray(list, padding);
 	recfree(list, free);
@@ -1004,7 +1004,7 @@ void print_subst_as_error(const paramexp_T *p)
 	    if (p->pe_type & PT_NEST)
 		xerror(0, "%ls", subst);
 	    else
-		xerror(0, "%s: %ls", p->pe_name, subst);
+		xerror(0, "%ls: %ls", p->pe_name, subst);
 	    free(subst);
 	}
     } else {
@@ -1012,8 +1012,8 @@ void print_subst_as_error(const paramexp_T *p)
 	    xerror(0, Ngt("parameter null"));
 	else
 	    xerror(0, (p->pe_type & PT_COLON)
-		    ? Ngt("%s: parameter null or not set")
-		    : Ngt("%s: parameter not set"),
+		    ? Ngt("%ls: parameter null or not set")
+		    : Ngt("%ls: parameter not set"),
 		    p->pe_name);
     }
 }
@@ -1538,7 +1538,7 @@ void fieldsplit_all(void **restrict valuelist, void **restrict splitlist,
 	plist_T *restrict dest)
 {
     void **s = valuelist, **t = splitlist;
-    const wchar_t *ifs = getvar(VAR_IFS);
+    const wchar_t *ifs = getvar(L VAR_IFS);
     if (!ifs)
 	ifs = DEFAULT_IFS;
     while (*s) {
