@@ -17,6 +17,7 @@
 
 
 #include "common.h"
+#include <assert.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -36,6 +37,23 @@
 #include "yash.h"
 
 
+typedef void setoptfunc_T(void *argp);
+struct setoptinfo_T {
+    setoptfunc_T *func;
+    void *argp;
+};
+
+static void set_ignore_option(void *argp);
+static void set_bool_option(void *argp);
+#if YASH_ENABLE_LINEEDIT
+static void set_yesnoauto_option(void *argp);
+#endif
+static void set_monitor_option(void *argp);
+#if YASH_ENABLE_LINEEDIT
+static void set_lineedit_option(void *argp);
+#endif
+
+
 /* If set, the shell behaves strictly as defined in POSIX.
  * Corresponds to --posix option. */
 bool posixly_correct;
@@ -56,6 +74,11 @@ bool do_job_control;
 /* If set, the shell immediately notifies on a change of job status.
  * Corresponds to -b/--notify option. */
 bool shopt_notify;
+#if YASH_ENABLE_LINEEDIT
+/* If set, the shell immediately notifies on a change of job status during line-
+ * editing. Ignored if `shopt_notify' is set. Corresponds to --notifyle option. */
+bool shopt_notifyle;
+#endif
 
 /* Set if commands are read from argument and stdin respectively.
  * Correspond to -c and -s options respectively. */
@@ -111,98 +134,241 @@ bool shopt_braceexpand;
  * Corresponds to -C/--noclobber option. */
 bool shopt_noclobber;
 
+#if YASH_ENABLE_LINEEDIT
+/* Type of line editing keybind. */
+enum shopt_lineedit_T shopt_lineedit = shopt_nolineedit;
+/* Defines treatment of the 8th bit of input characters. */
+enum shopt_yesnoauto_T shopt_le_convmeta = shopt_auto;
+/* If set, a special character sequence is printed when starting line-editing
+ * to make sure the prompt starts at the beginning of line. */
+bool shopt_le_promptsp = true;
+#endif
+
+
+typedef enum shopt_index_T {
+    SHOPT_INTERACTIVE, SHOPT_LOGIN, SHOPT_NOPROFILE, SHOPT_NORCFILE,
+    SHOPT_RCFILE, SHOPT_VERSION,
+    SHOPT_ALLEXPORT, SHOPT_HASHONDEF, SHOPT_NOCLOBBER, SHOPT_NOGLOB,
+    SHOPT_NOCASEGLOB, SHOPT_DOTGLOB, SHOPT_MARKDIRS, SHOPT_EXTENDEDGLOB,
+    SHOPT_NULLGLOB, SHOPT_BRACEEXPAND, SHOPT_CURASYNC, SHOPT_AUTOCD,
+    SHOPT_ERREXIT, SHOPT_NOUNSET, SHOPT_NOEXEC, SHOPT_IGNOREEOF, SHOPT_VERBOSE,
+    SHOPT_XTRACE, SHOPT_NOLOG, SHOPT_MONITOR, SHOPT_NOTIFY, SHOPT_NOTIFYLE,
+    SHOPT_POSIX, SHOPT_VI, /* SHOPT_EMACS, */ SHOPT_LE_CONVMETA,
+    SHOPT_LE_PROMPTSP, SHOPT_HELP,
+    SHOPT_setopt = SHOPT_ALLEXPORT,
+} shopt_index_T;
+
 
 /* Long options for the shell and set builtin */
 static const struct xoption long_options[] = {
-    { L"interactive",  xno_argument, L'i', },
-    { L"login",        xno_argument, L'l', },
-    { L"noprofile",    xno_argument, L'(', },
-    { L"norcfile",     xno_argument, L')', },
-    { L"rcfile",       xrequired_argument, L'!', },
-    { L"version",      xno_argument, L'V', },
+    [SHOPT_INTERACTIVE]  = { L"interactive",  xno_argument, L'i', },
+    [SHOPT_LOGIN]        = { L"login",        xno_argument, L'l', },
+    [SHOPT_NOPROFILE]    = { L"noprofile",    xno_argument, L'(', },
+    [SHOPT_NORCFILE]     = { L"norcfile",     xno_argument, L')', },
+    [SHOPT_RCFILE]       = { L"rcfile",       xrequired_argument, L'!', },
+    [SHOPT_VERSION]      = { L"version",      xno_argument, L'V', },
     /* Options above cannot be used in set builtin */
-    { L"allexport",    xno_argument, L'a', },
-    { L"hashondef",    xno_argument, L'h', },
-    { L"noclobber",    xno_argument, L'C', },
-    { L"noglob",       xno_argument, L'f', },
-    { L"nocaseglob",   xno_argument, L'S', },
-    { L"dotglob",      xno_argument, L'D', },
-    { L"markdirs",     xno_argument, L'M', },
-    { L"extendedglob", xno_argument, L'E', },
-    { L"nullglob",     xno_argument, L'N', },
-    { L"braceexpand",  xno_argument, L'B', },
-    { L"curasync",     xno_argument, L'Y', },
-    { L"autocd",       xno_argument, L'T', },
-    { L"errexit",      xno_argument, L'e', },
-    { L"nounset",      xno_argument, L'u', },
-    { L"noexec",       xno_argument, L'n', },
-    { L"ignoreeof",    xno_argument, L'I', },
-    { L"verbose",      xno_argument, L'v', },
-    { L"xtrace",       xno_argument, L'x', },
-    { L"nolog",        xno_argument, L'L', },
-    { L"monitor",      xno_argument, L'm', },
-    { L"notify",       xno_argument, L'b', },
-    { L"posix",        xno_argument, L'X', },
-    { L"help",         xno_argument, L'-', },
-    { NULL,            0,            0,   },
+    [SHOPT_ALLEXPORT]    = { L"allexport",    xno_argument, L'L', },
+    [SHOPT_HASHONDEF]    = { L"hashondef",    xno_argument, L'L', },
+    [SHOPT_NOCLOBBER]    = { L"noclobber",    xno_argument, L'L', },
+    [SHOPT_NOGLOB]       = { L"noglob",       xno_argument, L'L', },
+    [SHOPT_NOCASEGLOB]   = { L"nocaseglob",   xno_argument, L'L', },
+    [SHOPT_DOTGLOB]      = { L"dotglob",      xno_argument, L'L', },
+    [SHOPT_MARKDIRS]     = { L"markdirs",     xno_argument, L'L', },
+    [SHOPT_EXTENDEDGLOB] = { L"extendedglob", xno_argument, L'L', },
+    [SHOPT_NULLGLOB]     = { L"nullglob",     xno_argument, L'L', },
+    [SHOPT_BRACEEXPAND]  = { L"braceexpand",  xno_argument, L'L', },
+    [SHOPT_CURASYNC]     = { L"curasync",     xno_argument, L'L', },
+    [SHOPT_AUTOCD]       = { L"autocd",       xno_argument, L'L', },
+    [SHOPT_ERREXIT]      = { L"errexit",      xno_argument, L'L', },
+    [SHOPT_NOUNSET]      = { L"nounset",      xno_argument, L'L', },
+    [SHOPT_NOEXEC]       = { L"noexec",       xno_argument, L'L', },
+    [SHOPT_IGNOREEOF]    = { L"ignoreeof",    xno_argument, L'L', },
+    [SHOPT_VERBOSE]      = { L"verbose",      xno_argument, L'L', },
+    [SHOPT_XTRACE]       = { L"xtrace",       xno_argument, L'L', },
+    [SHOPT_NOLOG]        = { L"nolog",        xno_argument, L'L', },
+    [SHOPT_MONITOR]      = { L"monitor",      xno_argument, L'm', },
+    [SHOPT_NOTIFY]       = { L"notify",       xno_argument, L'L', },
+    [SHOPT_NOTIFYLE]     = { L"notifyle",     xno_argument, L'L', },
+    [SHOPT_POSIX]        = { L"posix",        xno_argument, L'L', },
+#if YASH_ENABLE_LINEEDIT
+    [SHOPT_VI]           = { L"vi",           xno_argument, L'L', },
+    // [SHOPT_EMACS]        = { L"emacs",        xno_argument, L'L', },
+    [SHOPT_LE_CONVMETA]  = { L"le-convmeta",  xrequired_argument, L'L', },
+    [SHOPT_LE_PROMPTSP]  = { L"le-promptsp",  xno_argument, L'L', },
+#endif
+    [SHOPT_HELP]         = { L"help",         xno_argument, L'-', },
+    /* this one must be the last for `help_option' */
+    { NULL, 0, 0, },
+};
+
+static const struct setoptinfo_T setoptinfo[] = {
+    [SHOPT_ALLEXPORT]    = { set_bool_option, &shopt_allexport, },
+    [SHOPT_HASHONDEF]    = { set_bool_option, &shopt_hashondef, },
+    [SHOPT_NOCLOBBER]    = { set_bool_option, &shopt_noclobber, },
+    [SHOPT_NOGLOB]       = { set_bool_option, &shopt_noglob, },
+    [SHOPT_NOCASEGLOB]   = { set_bool_option, &shopt_nocaseglob, },
+    [SHOPT_DOTGLOB]      = { set_bool_option, &shopt_dotglob, },
+    [SHOPT_MARKDIRS]     = { set_bool_option, &shopt_markdirs, },
+    [SHOPT_EXTENDEDGLOB] = { set_bool_option, &shopt_extendedglob, },
+    [SHOPT_NULLGLOB]     = { set_bool_option, &shopt_nullglob, },
+    [SHOPT_BRACEEXPAND]  = { set_bool_option, &shopt_braceexpand, },
+    [SHOPT_CURASYNC]     = { set_bool_option, &shopt_curasync, },
+    [SHOPT_AUTOCD]       = { set_bool_option, &shopt_autocd, },
+    [SHOPT_ERREXIT]      = { set_bool_option, &shopt_errexit, },
+    [SHOPT_NOUNSET]      = { set_bool_option, &shopt_nounset, },
+    [SHOPT_NOEXEC]       = { set_bool_option, &shopt_noexec, },
+    [SHOPT_IGNOREEOF]    = { set_bool_option, &shopt_ignoreeof, },
+    [SHOPT_VERBOSE]      = { set_bool_option, &shopt_verbose, },
+    [SHOPT_XTRACE]       = { set_bool_option, &shopt_xtrace, },
+    [SHOPT_NOLOG]        = { set_ignore_option, NULL, },
+    [SHOPT_MONITOR]      = { set_monitor_option, NULL, },
+    [SHOPT_NOTIFY]       = { set_bool_option, &shopt_notify, },
+#if YASH_ENABLE_LINEEDIT
+    [SHOPT_NOTIFYLE]     = { set_bool_option, &shopt_notifyle, },
+#endif
+    [SHOPT_POSIX]        = { set_bool_option, &posixly_correct, },
+#if YASH_ENABLE_LINEEDIT
+    [SHOPT_VI]           = { set_lineedit_option, NULL, },
+    //[SHOPT_EMACS]        = { set_lineedit_option, NULL, },
+    [SHOPT_LE_CONVMETA]  = { set_yesnoauto_option, &shopt_le_convmeta, },
+    [SHOPT_LE_PROMPTSP]  = { set_bool_option, &shopt_le_promptsp, },
+#endif
+    //[SHOPT_HELP]
+    { 0, NULL, },
 };
 
 const struct xoption *const shell_long_options = long_options;
-const struct xoption *const set_long_options   = long_options + 6;
-const struct xoption *const help_option
-    = long_options + ((sizeof long_options / sizeof *long_options) - 2);
+const struct xoption *const set_long_options   = long_options + SHOPT_setopt;
+const struct xoption *const help_option        = long_options + SHOPT_HELP;
 
-// TODO option: unimplemented options: -o{vi,emacs}
+/* The index of the currently parsing option in the `long_options' and
+ * `setoptinfo' arrays. */
+int setoptindex;
 
 
-/* Switches a one-character option depending on whether `xoptopt' is '-' or not.
- * Option characters that can only be used in shell invokation are ignored. */
-void set_option(wchar_t c)
+/* Sets the option specified by `setoptindex'. */
+void set_option(void)
 {
-    bool value = (xoptopt == L'-');
-    switch (c) {
-	case L'a':   shopt_allexport    = value;   break;
-	case L'h':   shopt_hashondef    = value;   break;
-	case L'C':   shopt_noclobber    = value;   break;
-	case L'f':   shopt_noglob       = value;   break;
-	case L'S':   shopt_nocaseglob   = value;   break;
-	case L'D':   shopt_dotglob      = value;   break;
-	case L'M':   shopt_markdirs     = value;   break;
-	case L'E':   shopt_extendedglob = value;   break;
-	case L'N':   shopt_nullglob     = value;   break;
-	case L'B':   shopt_braceexpand  = value;   break;
-	case L'Y':   shopt_curasync     = value;   break;
-	case L'T':   shopt_autocd       = value;   break;
-	case L'e':   shopt_errexit      = value;   break;
-	case L'u':   shopt_nounset      = value;   break;
-	case L'n':   shopt_noexec       = value;   break;
-	case L'I':   shopt_ignoreeof    = value;   break;
-	case L'v':   shopt_verbose      = value;   break;
-	case L'x':   shopt_xtrace       = value;   break;
-	case L'L':   /* XXX nolog unsupported */   break;
-	case L'm':   do_job_control     = value;   break;
-	case L'b':   shopt_notify       = value;   break;
-	case L'X':   posixly_correct    = value;   break;
-    }
+    const struct setoptinfo_T *info = &setoptinfo[setoptindex];
+    info->func(info->argp);
 }
 
-/* Switches the setting of a specified long option according to `xoptopt'.
- * Options that can only be used in shell invokation are ignored.
+/* Sets the option specified by the character `c'.
+ * An unrecognized character is ignored. */
+void set_single_option(wchar_t c)
+{
+    switch (c) {
+	case L'a':  setoptindex = SHOPT_ALLEXPORT;  break;
+	case L'h':  setoptindex = SHOPT_HASHONDEF;  break;
+	case L'C':  setoptindex = SHOPT_NOCLOBBER;  break;
+	case L'f':  setoptindex = SHOPT_NOGLOB;     break;
+	case L'e':  setoptindex = SHOPT_ERREXIT;    break;
+	case L'u':  setoptindex = SHOPT_NOUNSET;    break;
+	case L'n':  setoptindex = SHOPT_NOEXEC;     break;
+	case L'v':  setoptindex = SHOPT_VERBOSE;    break;
+	case L'x':  setoptindex = SHOPT_XTRACE;     break;
+	case L'm':  setoptindex = SHOPT_MONITOR;    break;
+	case L'b':  setoptindex = SHOPT_NOTIFY;     break;
+	default:    return;
+    }
+    set_option();
+}
+
+/* Switches the setting of a specified long option.
+ * Options that can only be used in shell invocation are ignored.
  * Returns true if successful, false for invalid options. */
 bool set_long_option(const wchar_t *s)
 {
-    const struct xoption *opt = set_long_options;
-
-    while (opt->name) {
-	if (wcscmp(s, opt->name) == 0) {
-	    set_option(opt->val);
-	    return true;
+    for (const struct xoption *optp = set_long_options; optp->name; optp++) {
+	if (wcscmp(s, optp->name) == 0) {
+	    if (optp->has_arg == xno_argument) {
+		setoptindex = optp - long_options;
+		set_option();
+		return true;
+	    } else {
+		return false;
+	    }
 	}
-	opt++;
     }
     return false;
 }
 
+/* This function does nothing. */
+void set_ignore_option(void *argp __attribute__((unused))) { }
+
+/* Changes the setting of a boolean option according to the current `xoptopt'.
+ * `argp' must be a pointer to a boolean value. */
+void set_bool_option(void *argp)
+{
+    bool *optp = argp;
+    *optp = (xoptopt == L'-');
+}
+
+#if YASH_ENABLE_LINEEDIT
+
+/* Changes the setting of a yes-no-auto option according to the current `xoptarg'.
+ * `argp' must be a pointer to a yes-no-auto value. */
+void set_yesnoauto_option(void *argp)
+{
+    enum shopt_yesnoauto_T *optp = argp;
+    
+    assert(xoptarg != NULL);
+    switch (xoptarg[0]) {
+	case L'Y':  case L'y':  *optp = shopt_yes;   break;
+	case L'N':  case L'n':  *optp = shopt_no;    break;
+	case L'A':  case L'a':  *optp = shopt_auto;  break;
+    }
+}
+
+#endif
+
+/* Changes the setting of the `-m' (--monitor) option.
+ * This function's behavior depends on the value of `shell_pid'. */
+void set_monitor_option(void *argp __attribute__((unused)))
+{
+    bool newvalue = (xoptopt == L'-');
+    if (shell_pid == 0) {
+	do_job_control = newvalue;
+    } else {
+	if (newvalue != do_job_control) {
+	    reset_own_pgid();
+	    do_job_control = newvalue;
+	    if (newvalue && ttyfd < 0)
+		open_ttyfd();
+	    set_signals();
+	    set_own_pgid();
+	}
+    }
+    /* When `shell_pid' is zero, the shell is under initialization;
+     * `set_own_pgid' and other functions are called later in the `main'
+     * function. */
+}
+
+#if YASH_ENABLE_LINEEDIT
+
+/* Changes the setting of the vi/emacs option. */
+void set_lineedit_option(void *argp __attribute__((unused)))
+{
+    enum shopt_lineedit_T opt;
+
+    switch (setoptindex) {
+	case SHOPT_VI:     opt = shopt_vi;     break;
+	//case SHOPT_EMACS:  opt = shopt_emacs;  break;
+	default:           return;
+    }
+    if (xoptopt == L'-') {
+	shopt_lineedit = opt;
+	/* turn on lineedit */
+    } else {
+	/* turn off lineedit */
+	if (shopt_lineedit == opt)
+	    shopt_lineedit = shopt_nolineedit;
+    }
+}
+
+#endif /* YASH_ENABLE_LINEEDIT */
+
 /* Return current value of special parameter $- as a newly malloced string. */
 wchar_t *get_hyphen_parameter(void)
 {
@@ -251,10 +417,16 @@ int set_builtin(int argc, void **argv)
     }
 
     xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(
-		    argv, L"+*o:" SHELLSET_OPTIONS, set_long_options, NULL)))
+    while ((opt = xgetopt_long(argv,
+		    L"+*o:" SHELLSET_OPTIONS,
+		    set_long_options,
+		    &setoptindex)))
     {
 	switch (opt) {
+	    case L'L':
+		setoptindex += SHOPT_setopt;
+		set_option();
+		break;
 	    case L'o':
 		if (!set_long_option(xoptarg)) {
 		    xerror(0, Ngt("%lco %ls: invalid option"),
@@ -272,19 +444,7 @@ int set_builtin(int argc, void **argv)
 		SPECIAL_BI_ERROR;
 		return Exit_ERROR;
 	    default:
-		if (opt == L'm') {
-		    bool newvalue = (xoptopt == L'-');
-		    if (newvalue != do_job_control) {
-			reset_own_pgid();
-			do_job_control = newvalue;
-			if (newvalue && ttyfd < 0)
-			    open_ttyfd();
-			set_signals();
-			set_own_pgid();
-		    }
-		} else {
-		    set_option(opt);
-		}
+		set_single_option(opt);
 		break;
 	}
     }
@@ -299,20 +459,33 @@ int set_builtin(int argc, void **argv)
 
 void set_builtin_print_current_settings(void)
 {
-    const char *yes = gt("yes"), *no = gt("no");
+    const char *vals[] = {
+	[shopt_yes]  = gt("yes"),
+	[shopt_no]   = gt("no"),
+	[shopt_auto] = gt("auto"),
+    };
 #define PRINTSETTING(name,value) \
-    printf("%-15ls %s\n", L"" #name, (value) ? yes : no)
+    printf("%-15ls %s\n", L"" #name, (value) ? vals[shopt_yes] : vals[shopt_no])
+#define PRINTSETTING_YNA(name,value) \
+    printf("%-15ls %s\n", L"" #name, vals[value])
 
     PRINTSETTING(allexport, shopt_allexport);
     PRINTSETTING(autocd, shopt_autocd);
     PRINTSETTING(braceexpand, shopt_braceexpand);
     PRINTSETTING(curasync, shopt_curasync);
     PRINTSETTING(dotglob, shopt_dotglob);
+#if YASH_ENABLE_LINEEDIT
+    //PRINTSETTING(emacs, shopt_lineedit == shopt_emacs);
+#endif
     PRINTSETTING(errexit, shopt_errexit);
     PRINTSETTING(extendedglob, shopt_extendedglob);
     PRINTSETTING(hashondef, shopt_hashondef);
     PRINTSETTING(ignoreeof, shopt_ignoreeof);
     PRINTSETTING(interactive, is_interactive);
+#if YASH_ENABLE_LINEEDIT
+    PRINTSETTING_YNA(le-convmeta, shopt_le_convmeta);
+    PRINTSETTING(le-promptsp, shopt_le_promptsp);
+#endif
     PRINTSETTING(login, is_login_shell);
     PRINTSETTING(markdirs, shopt_markdirs);
     PRINTSETTING(monitor, do_job_control);
@@ -322,12 +495,19 @@ void set_builtin_print_current_settings(void)
     PRINTSETTING(noglob, shopt_noglob);
     //PRINTSETTING(nolog, shopt_nolog);
     PRINTSETTING(notify, shopt_notify);
+#if YASH_ENABLE_LINEEDIT
+    PRINTSETTING(notifyle, shopt_notifyle);
+#endif
     PRINTSETTING(nounset, shopt_nounset);
     PRINTSETTING(nullglob, shopt_nullglob);
     PRINTSETTING(posix, posixly_correct);
     PRINTSETTING(verbose, shopt_verbose);
+#if YASH_ENABLE_LINEEDIT
+    PRINTSETTING(vi, shopt_lineedit == shopt_vi);
+#endif
     PRINTSETTING(xtrace, shopt_xtrace);
 #undef PRINTSETTING
+#undef PRINTSETTING_YNA
 }
 
 void set_builtin_print_restoring_commands(void)
@@ -340,11 +520,17 @@ void set_builtin_print_restoring_commands(void)
     PRINTSETTING(braceexpand, shopt_braceexpand);
     PRINTSETTING(curasync, shopt_curasync);
     PRINTSETTING(dotglob, shopt_dotglob);
+#if YASH_ENABLE_LINEEDIT
+    //PRINTSETTING(emacs, shopt_lineedit == shopt_emacs);
+#endif
     PRINTSETTING(errexit, shopt_errexit);
     PRINTSETTING(extendedglob, shopt_extendedglob);
     PRINTSETTING(hashondef, shopt_hashondef);
     PRINTSETTING(ignoreeof, shopt_ignoreeof);
     //PRINTSETTING(interactive, is_interactive);
+#if YASH_ENABLE_LINEEDIT
+    PRINTSETTING(le-promptsp, shopt_le_promptsp);
+#endif
     //PRINTSETTING(login, is_login_shell);
     PRINTSETTING(markdirs, shopt_markdirs);
     PRINTSETTING(monitor, do_job_control);
@@ -354,10 +540,16 @@ void set_builtin_print_restoring_commands(void)
     PRINTSETTING(noglob, shopt_noglob);
     //PRINTSETTING(nolog, shopt_nolog);
     PRINTSETTING(notify, shopt_notify);
+#if YASH_ENABLE_LINEEDIT
+    PRINTSETTING(notifyle, shopt_notifyle);
+#endif
     PRINTSETTING(nounset, shopt_nounset);
     PRINTSETTING(nullglob, shopt_nullglob);
     PRINTSETTING(posix, posixly_correct);
     PRINTSETTING(verbose, shopt_verbose);
+#if YASH_ENABLE_LINEEDIT
+    PRINTSETTING(vi, shopt_lineedit == shopt_vi);
+#endif
     PRINTSETTING(xtrace, shopt_xtrace);
 #undef PRINTSETTING
 }
@@ -367,10 +559,11 @@ const char set_help[] = Ngt(
 "\tset [-abefhmnuvxC] [+abefhmnuvxC] [-o option] [+o option] [--] [arg...]\n"
 "\tset -o\n"
 "\tset +o\n"
-"The first form sets the specified shell options and positional parameters to\n"
-"<arg>s. If no <arg>s are given, the positional parameters are unchanged.\n"
+"The first form enables or disables the specified shell options and/or sets\n"
+"positional parameters to <arg>s. If no <arg>s are given, the positional\n"
+"parameters are not changed.\n"
 "If no options or <arg>s are given at all, a list of all variables currently\n"
-"defined is printed. To remove all the current positional parameters, use\n"
+"defined in the shell is printed. To clear all positional parameters, use\n"
 "`set --'.\n"
 "The second form prints the current settings of the shell options in a human-\n"
 "readable form.\n"
@@ -381,6 +574,9 @@ const char set_help[] = Ngt(
 "\tAny variable is exported when assigned.\n"
 " -b --notify\n"
 "\tWhen the status of a job is changed, it is notified immediately.\n"
+" --notifyle\n"
+"\tSimilar to -b, but only notify during line-editing. Ignored if -b is\n"
+"\tset.\n"
 " -e --errexit\n"
 "\tExit the shell immediately when any simple command returns a\n"
 "\tnon-zero status.\n"
@@ -389,8 +585,8 @@ const char set_help[] = Ngt(
 " --nocaseglob\n"
 "\tPerform pathname expansion case-insensitively.\n"
 " --dotglob\n"
-"\tIn pathname expansion, '*' and '?' match a '.' at the start of the\n"
-"\tfilename.\n"
+"\tIn pathname expansion, '*' and '?' match a '.' at the beginning of\n"
+"\tthe filename.\n"
 " --markdirs\n"
 "\tIn pathname expansion, pathnames expanded to directories have a '/'\n"
 "\tat the end of the name.\n"
@@ -401,7 +597,7 @@ const char set_help[] = Ngt(
 "\tremoved from the command line rather than left as is.\n"
 " -h --hashondef\n"
 "\tWhen a function is defined, all the commands in the function are\n"
-"\tregistered in the command name hashtable.\n"
+"\tregistered in the command path cache.\n"
 " -m --monitor\n"
 "\tEnable job control. All jobs are run in their own process group.\n"
 "\tWhen the status of a job is changed, the status is reported before\n"
@@ -431,9 +627,17 @@ const char set_help[] = Ngt(
 " --autocd\n"
 "\tIf a simple command cannot be executed but it is a directory name,\n"
 "\tthe shell performs the `cd' command to that directory.\n"
-"\t\n"
 " --posix\n"
 "\tMake the shell behave as the POSIX shell.\n"
+" --vi\n"
+"\tEnable vi-like editing.\n"
+//" --emacs\n"
+//"\tEnable emacs-like editing.\n"
+" --le-convmeta=<yes|no|auto>\n"
+"\tTreat 8th bit of input as a meta-key flag.\n"
+" --le-promptsp\n"
+"\tMove cursor to beginning of line each time when starting\n"
+"\tline-editing. (enabled by default)\n"
 "To disable options, put '+' before the option characters instead of '-'.\n"
 "Long options in the form of `--xxx' are equivalent to `-o xxx'.\n"
 "Use `+o xxx' to turn off a long option. You cannot use `+-xxx' or `++xxx'.\n"
