@@ -75,7 +75,7 @@ typedef struct command_T {
 	struct and_or_T     *subcmds;  /* contents of command group */
 	struct ifcommand_T  *ifcmds;   /* contents of if command */
 	struct {
-	    char            *forname;  /* loop variable of for loop */
+	    wchar_t         *forname;  /* loop variable of for loop */
 	    void           **forwords; /* words assigned to loop variable */
 	    struct and_or_T *forcmds;  /* commands executed in for loop */
 	} forcontent;
@@ -89,7 +89,7 @@ typedef struct command_T {
 	    struct caseitem_T *casitems;  /* pairs of patterns and commands */
 	} casecontent;
 	struct {
-	    char             *funcname;  /* name of function */
+	    wchar_t          *funcname;  /* name of function */
 	    struct command_T *funcbody;  /* body of function */
 	} funcdef;
     } c_content;
@@ -199,7 +199,7 @@ typedef enum {
 typedef struct paramexp_T {
     paramexptype_T pe_type;
     union {
-	char              *name;
+	wchar_t           *name;
 	struct wordunit_T *nest;
     } pe_value;
     struct wordunit_T *pe_start, *pe_end;
@@ -225,7 +225,7 @@ typedef enum {
 typedef struct assign_T {
     struct assign_T *next;
     assigntype_T a_type;
-    char *a_name;
+    wchar_t *a_name;
     union {
 	struct wordunit_T *scalar;
 	void **array;          
@@ -306,14 +306,11 @@ typedef struct parseinfo_T {
     inputfunc_T *input;   /* input function */
     void *inputinfo;      /* pointer passed to the input function */
     _Bool intrinput;      /* input is interactive? */
-    _Bool inputisatty;    /* input is from interactive terminal? */
     int lastinputresult;  /* last return value of input function */
 } parseinfo_T;
 /* If `intrinput' is true, `input' is `input_readline' and `inputinfo' is a
- * pointer to a `struct input_readline' object.
- * Note that input may not be from a terminal even if `intrinput' is true.
- * Iff `intrinput' is true and input is actually from a terminal, `inputisatty'
- * is true. */
+ * pointer to a `struct input_readline_info' object.
+ * Note that input may not be from a terminal even if `intrinput' is true. */
 
 extern struct parsestate_T *save_parse_state(void)
     __attribute__((malloc,warn_unused_result));
@@ -332,6 +329,10 @@ extern _Bool parse_string(
 
 /********** Auxiliary function **********/
 
+extern _Bool is_name_char(wchar_t c)
+    __attribute__((pure));
+extern _Bool is_name(const wchar_t *s)
+    __attribute__((pure));
 extern _Bool is_keyword(const wchar_t *s)
     __attribute__((nonnull,pure));
 extern _Bool is_token_delimiter_char(wchar_t c)
