@@ -1066,13 +1066,13 @@ int cd_builtin(int argc, void **argv)
     }
 
     if (argc <= xoptind) {  /* step 1-2 */
-	newpwd = getvar(VAR_HOME);
+	newpwd = getvar(L VAR_HOME);
 	if (newpwd == NULL || newpwd[0] == L'\0') {
 	    xerror(0, Ngt("$HOME not set"));
 	    return Exit_FAILURE;
 	}
     } else if (wcscmp(ARGV(xoptind), L"-") == 0) {
-	newpwd = getvar(VAR_OLDPWD);
+	newpwd = getvar(L VAR_OLDPWD);
 	if (newpwd == NULL || newpwd[0] == L'\0') {
 	    xerror(0, Ngt("$OLDPWD not set"));
 	    return Exit_FAILURE;
@@ -1098,7 +1098,7 @@ int change_directory(const wchar_t *newpwd, bool printnewdir, bool logical)
     bool err = false;
 
     /* get the current value of $PWD as `oldpwd' */
-    oldpwd = getvar(VAR_PWD);
+    oldpwd = getvar(L VAR_PWD);
     if (oldpwd == NULL || oldpwd[0] != L'/') {
 	if (oldpwd == newpwd) {
 	    xerror(0, Ngt("invalid $PWD value"));
@@ -1114,8 +1114,8 @@ int change_directory(const wchar_t *newpwd, bool printnewdir, bool logical)
 	} else {
 	    wchar_t *wpwd = realloc_mbstowcs(pwd);
 	    if (wpwd != NULL) {
-		if (set_variable(VAR_PWD, wpwd, SCOPE_GLOBAL, false))
-		    oldpwd = getvar(VAR_PWD);
+		if (set_variable(L VAR_PWD, wpwd, SCOPE_GLOBAL, false))
+		    oldpwd = getvar(L VAR_PWD);
 		else
 		    logical = false, oldpwd = NULL;
 	    } else {
@@ -1145,7 +1145,7 @@ int change_directory(const wchar_t *newpwd, bool printnewdir, bool logical)
 	char *mbsnewpwd = malloc_wcstombs(newpwd);
 	if (mbsnewpwd == NULL) {
 	    wb_destroy(&curpath);
-	    xerror(0, Ngt("unexpected error"));
+	    xerror(EILSEQ, Ngt("unexpected error"));
 	    return Exit_ERROR;
 	}
 	char *path = which(mbsnewpwd, get_path_array(PA_CDPATH), is_directory);
@@ -1230,12 +1230,12 @@ step10:  /* do chdir */
 
     /* set $OLDPWD and $PWD */
     if (oldpwd != NULL)
-	if (!set_variable(VAR_OLDPWD, xwcsdup(oldpwd), SCOPE_GLOBAL, false))
+	if (!set_variable(L VAR_OLDPWD, xwcsdup(oldpwd), SCOPE_GLOBAL, false))
 	    err = true;
     if (logical) {
 	if (printnewdir)
 	    printf("%ls\n", curpath.contents);
-	if (!set_variable(VAR_PWD, wb_towcs(&curpath), SCOPE_GLOBAL, false))
+	if (!set_variable(L VAR_PWD, wb_towcs(&curpath), SCOPE_GLOBAL, false))
 	    err = true;
     } else {
 	wb_destroy(&curpath);
@@ -1250,11 +1250,11 @@ step10:  /* do chdir */
 
 	    wchar_t *wnewpwd = realloc_mbstowcs(mbsnewpwd);
 	    if (wnewpwd == NULL) {
-		xerror(0, Ngt("cannot convert multibyte characters "
+		xerror(EILSEQ, Ngt("cannot convert multibyte characters "
 			    "into wide characters"));
 		err = true;
 	    } else {
-		if (!set_variable(VAR_PWD, wnewpwd, SCOPE_GLOBAL, false))
+		if (!set_variable(L VAR_PWD, wnewpwd, SCOPE_GLOBAL, false))
 		    err = true;
 	    }
 	}
@@ -1276,7 +1276,7 @@ const char cd_help[] = Ngt(
 "When the -L (--logical) option is specified, symbolic links in $PWD are left\n"
 "unchanged. When the -P (--physical) option is specified, symbolic links are\n"
 "resolved so that $PWD does not contain any symbolic links.\n"
-"-L and -P are mutually exclusive: the one specified last is used.\n"
+"-L and -P are mutually exclusive: the last specified one is used.\n"
 "If neither is specified, -L is the default.\n"
 );
 
@@ -1304,7 +1304,7 @@ int pushd_builtin(int argc __attribute__((unused)), void **argv)
 	}
     }
 
-    const wchar_t *oldpwd = getvar(VAR_PWD);
+    const wchar_t *oldpwd = getvar(L VAR_PWD);
     if (!oldpwd) {
 	xerror(0, Ngt("$PWD not set"));
 	return Exit_FAILURE;
@@ -1319,7 +1319,7 @@ int pushd_builtin(int argc __attribute__((unused)), void **argv)
     if (wcscmp(arg, L"-") == 0) {
 	printnewdir = true;
 	stackindex = SIZE_MAX;
-	newpwd = getvar(VAR_OLDPWD);
+	newpwd = getvar(L VAR_OLDPWD);
 	if (!newpwd) {
 	    xerror(0, Ngt("$OLDPWD not set"));
 	    return Exit_FAILURE;
@@ -1450,7 +1450,7 @@ int pwd_builtin(int argc __attribute__((unused)), void **argv)
     }
 
     if (logical) {
-	const wchar_t *pwd = getvar(VAR_PWD);
+	const wchar_t *pwd = getvar(L VAR_PWD);
 	if (pwd != NULL && pwd[0] == L'/' && is_normalized_path(pwd)) {
 	    mbspwd = malloc_wcstombs(pwd);
 	    if (mbspwd != NULL) {
@@ -1482,10 +1482,8 @@ const char pwd_help[] = Ngt(
 "correct. It may contain symbolic links in the pathname.\n"
 "When the -P (--physical) option is specified, the printed pathname does not\n"
 "contain any symbolic links.\n"
-"-L and -P are mutually exclusive: the one specified last is used.\n"
+"-L and -P are mutually exclusive: the last specified one is used.\n"
 "If neither is specified, -L is the default.\n"
-"If the shell is in POSIXly correct mode and the -P option is specified,\n"
-"$PWD is set to the printed pathname.\n"
 );
 
 /* The "hash" builtin, which accepts the following options:
