@@ -45,6 +45,7 @@
 #include "strbuf.h"
 #include "util.h"
 #include "yash.h"
+#include "lineedit/lineedit.h"
 
 
 /* About the shell's signal handling:
@@ -252,6 +253,9 @@ void init_signal(void)
 	sigemptyset(&blocked_signals);
 	sigemptyset(&ss);
 	sigaddset(&ss, SIGCHLD);
+#if YASH_ENABLE_LINEEDIT && defined(SIGWINCH)
+	sigaddset(&ss, SIGWINCH);
+#endif
 	if (sigprocmask(SIG_UNBLOCK, &ss, &blocked_signals) < 0)
 	    xerror(errno, "sigprocmask");
 	set_special_handler(SIGCHLD, sig_handler);
@@ -571,10 +575,17 @@ void handle_sigchld(void)
     if (sigchld_received) {
         sigchld_received = false;
         do_wait();
+#if YASH_ENABLE_LINEEDIT
+	if (shopt_notify || (shopt_notifyle && le_state == LE_STATE_ACTIVE)) {
+	    le_suspend_readline();
+	    print_job_status_all(true, false, stderr);
+	    le_resume_readline();
+	}
+#else
 	if (shopt_notify) {
 	    print_job_status_all(true, false, stderr);
-	    // XXX redisplay prompt if lineedit is active
 	}
+#endif
     }
 }
 
@@ -617,6 +628,10 @@ int handle_traps(void)
     if (!any_trap_set || !any_signal_received || handled_signal >= 0)
 	return false;
 
+#if YASH_ENABLE_LINEEDIT
+    le_suspend_readline();
+#endif
+
     int signum = 0;
     struct parsestate_T *state = NULL;
     savelaststatus = laststatus;
@@ -661,6 +676,13 @@ exec_handlers:
 	}
     }
 #endif
+
+#if YASH_ENABLE_LINEEDIT
+    if (shopt_notifyle && le_state == LE_STATE_SUSPENDED)
+	print_job_status_all(true, false, stderr);
+    le_resume_readline();
+#endif
+
     if (any_signal_received)
 	goto exec_handlers;
     savelaststatus = -1;
@@ -1000,13 +1022,16 @@ void print_trap(const char *signame, const wchar_t *command)
 const char trap_help[] = Ngt(
 "trap - set signal handler\n"
 "\ttrap [action signal...]\n"
+"\ttrap signum [signal...]\n"
 "\ttrap -p [signal...]\n"
 "Sets the signal handler of the specified <signal>s to <action>.\n"
 "When the shell receives the signal, the corresponding action is executed as\n"
-"if by the \"exec\" command.\n"
+"if by the \"eval\" command.\n"
 "If <action> is an empty string, no actions are taken and the signal is\n"
 "silently ignored when the signal is received.\n"
 "If <action> is \"-\", the signal handler is reset to the default.\n"
+"If the first operand is a non-negative integer, the operand is considered as\n"
+"a signal specification and <action> is assumed to be \"-\".\n"
 "If the -p (--print) option is specified, the actions for the specified\n"
 "<signal>s are printed.\n"
 "Without any operands, all signal handlers currently set are printed.\n"
