@@ -41,16 +41,13 @@ extern void restore_interactive_signals(void);
 extern void set_interruptible_by_sigint(_Bool onoff);
 extern void ignore_sigquit_and_sigint(void);
 extern void ignore_sigtstp(void);
-extern void block_sigttou(void);
-extern void unblock_sigttou(void);
 extern _Bool send_sigstop_to_myself(void);
 
-extern void block_sigchld_and_sigint(void);
-extern void unblock_sigchld_and_sigint(void);
 extern int wait_for_sigchld(_Bool interruptible, _Bool return_on_trap);
 extern void wait_for_input(int fd, _Bool trap);
 
 extern void handle_sigchld(void);
+extern _Bool have_unhandled_traps(void);
 extern int handle_traps(void);
 extern void execute_exit_trap(void);
 extern void clear_traps(void);
